@@ -160,4 +160,9 @@ $(function () {
       sandbox.restore()
     }
   })
+
+  QUnit.test('Util.jQueryDetection should detect jQuery', function (assert) {
+    assert.expect(0)
+    Util.jQueryDetection()
+  })
 })
